"""
File       : MongoDB.py
Description: Provides a wrapper class for MongoDB
"""

# futures
from __future__ import division, print_function
from builtins import str, object

try:
    import mongomock
except ImportError:
    # this library should only be required by unit tests
    mongomock = None

from pymongo import MongoClient, errors, IndexModel
from pymongo.errors import ConnectionFailure


class MongoDB(object):
    """
    A simple wrapper class for creating a connection to a MongoDB instance
    """
    def __init__(self, database=None, server=None,
                 create=False, collections=None, testIndexes=False,
                 logger=None, mockMongoDB=False, **kwargs):
        """
        :databases:   A database Name to connect to
        :server:      The server url or a list of (server:port) pairs (see https://docs.mongodb.com/manual/reference/connection-string/)
        :create:      A flag to trigger a database creation (if missing) during
                      object construction, together with collections if present.
        :collections: A list of tuples describing collections with indexes -
                      the first element is considered the collection name, all
                      the rest elements are considered as indexes
        :testIndexes: A flag to trigger index test and eventually to create them
                      if missing (TODO)
        :mockMongoDB: A flag to trigger a database simulation instead of trying
                      to connect to a real database server.
        :logger:      Logger

        Here follows a short list of usefull optional parameters accepted by the
        MongoClient which may be passed as keyword arguments to the current module:

        :replicaSet:       The name of the replica set to connect to. The driver will verify
                           that all servers it connects to match this name. Implies that the
                           hosts specified are a seed list and the driver should attempt to
                           find all members of the set. Defaults to None.
        :port:             The port number on which to connect. It is overwritten by the ports
                           defined in the Url string or from the tuples listed in the server list
        :connect:          If True, immediately begin connecting to MongoDB in the background.
                           Otherwise connect on the first operation.
        :directConnection: If True, forces the client to connect directly to the specified MongoDB
                           host as a standalone. If False, the client connects to the entire
                           replica set of which the given MongoDB host(s) is a part.
                           If this is True and a mongodb+srv:// URI or a URI containing multiple
                           seeds is provided, an exception will be raised.
        :username:         A string
        :password:         A string
                           Although username and password must be percent-escaped in a MongoDB URI,
                           they must not be percent-escaped when passed as parameters. In this example,
                           both the space and slash special characters are passed as-is:
                           MongoClient(username="user name", password="pass/word")
        """
        self.server = server
        self.logger = logger
        self.mockMongoDB = mockMongoDB
        if mockMongoDB and mongomock is None:
            msg = "You are trying to mock MongoDB, but you do not have mongomock in the python path."
            self.logger.critical(msg)
            raise ImportError(msg)

        # NOTE: We need to explicitely check for server availiability.
        #       From pymongo Documentation: https://pymongo.readthedocs.io/en/stable/api/pymongo/mongo_client.html
        #       """
        #           ...
        #           Starting with version 3.0 the :class:`MongoClient`
        #           constructor no longer blocks while connecting to the server or
        #           servers, and it no longer raises
        #           :class:`~pymongo.errors.ConnectionFailure` if they are
        #           unavailable, nor :class:`~pymongo.errors.ConfigurationError`
        #           if the user's credentials are wrong. Instead, the constructor
        #           returns immediately and launches the connection process on
        #           background threads.
        #           ...
        #       """
        try:
            if mockMongoDB:
                self.client = mongomock.MongoClient()
                self.logger.info("NOTICE: MongoDB is set to use mongomock, instead of real database.")
            else:
                self.client = MongoClient(host=self.server, **kwargs)
            self.client.server_info()
            self.client.admin.command('ping')
        except ConnectionFailure as ex:
            msg = "Could not connect to MongoDB server: %s. Server not available. \n"
            msg += "Giving up Now."
            self.logger.error(msg, self.server)
            raise ex from None
        except Exception as ex:
            msg = "Could not connect to MongoDB server: %s. Due to unknown reason: %s\n"
            msg += "Giving up Now."
            self.logger.error(msg, self.server, str(ex))
            raise ex from None
        self.create = create
        self.testIndexes = testIndexes
        self.dbName = database
        self.collections = collections or []

        self._dbConnect(database)

        if self.create and self.collections:
            for collection in self.collections:
                self._collCreate(collection, database)

        if self.testIndexes and self.collections:
            for collection in self.collections:
                self._indexTest(collection[0], collection[1])

    def _indexTest(self, collection, index):
        pass

    def _collTest(self, coll, db):
        # self[db].list_collection_names()
        pass

    def collCreate(self, coll):
        """
        A public method for _collCreate
        """
        self._collCreate(coll, self.database)

    def _collCreate(self, coll, db):
        """
        A function used to explicitly create a collection with the relevant
        indexes - used to avoid the Lazy Creating from MongoDB and eventual issues
        in case we end up with no indexed collection, especially ones missing
        the (`unique` index parameter)
        :coll: A tuple describing one collection with indexes -
               The first element is considered to be the collection name, and all
               the rest of the elements are considered to be indexes.
               The indexes must be of type IndexModel. See pymongo documentation:

               https://api.mongodb.com/python/current/api/pymongo/collection.html#pymongo.collection.Collection.create_index

        :db:   The database name for the collection
        """

        collName = coll[0]
        collIndexes = list(coll[1:])
        try:
            self.client[db].create_collection(collName)
        except errors.CollectionInvalid:
            # this error is thrown in case of an already existing collection
            msg = "Collection '{}' Already exists in database '{}'".format(coll, db)
            self.logger.warning(msg)

        if collIndexes:
            for index in collIndexes:
                if not isinstance(index, IndexModel):
                    msg = "ERR: Bad Index type for collection %s" % collName
                    raise errors.InvalidName
            try:
                self.client[db][collName].create_indexes(collIndexes)
            except Exception as ex:
                msg = "Failed to create indexes on collection: %s\n%s" % (collName, str(ex))
                self.logger.error(msg)
                raise ex

    def _dbTest(self, db):
        """
        Tests database connection.
        """
        # Test connection (from mongoDB documentation):
        # https://api.mongodb.com/python/3.4.0/api/pymongo/mongo_client.html
        try:
            # The 'ismaster' command is cheap and does not require auth.
            self.client.admin.command('ismaster')
        except errors.ConnectionFailure as ex:
            msg = "Server not available: %s" % str(ex)
            self.logger.error(msg)
            raise ex

        # Test for database existence
        if db not in self.client.list_database_names():
            msg = "Missing MongoDB databases: %s" % db
            self.logger.error(msg)
            raise errors.InvalidName

    def _dbCreate(self, db):
        # creating an empty collection in order to create the database
        _initColl = self.client[db].create_collection('_initCollection')
        _initColl.insert_one({})
        # NOTE: never delete the _initCollection if you want the database to persist
        # self.client[db].drop_collection('_initCollection')

    def dbConnect(self):
        """
        A public method for _dbConnect
        """
        self._dbConnect(self.database)

    def _dbConnect(self, db):
        """
        The function to be used for the initial database connection creation and testing
        """
        try:
            setattr(self, db, self.client[db])
            if not self.mockMongoDB:
                self._dbTest(db)
        except errors.ConnectionFailure as ex:
            msg = "Could not connect to MongoDB server for database: %s\n%s\n" % (db, str(ex))
            msg += "Giving up Now."
            self.logger.error(msg)
            raise ex
        except errors.InvalidName as ex:
            msg = "Could not connect to a missing MongoDB databases: %s\n%s" % (db, str(ex))
            self.logger.error(msg)
            if self.create:
                msg = "Trying to create: %s" % db
                self.logger.error(msg)
                try:
                    # self._dbCreate(getattr(self, db))
                    self._dbCreate(db)
                except Exception as exc:
                    msg = "Could not create MongoDB databases: %s\n%s\n" % (db, str(exc))
                    msg += "Giving up Now."
                    self.logger.error(msg)
                    raise exc
                try:
                    self._dbTest(db)
                except Exception as exc:
                    msg = "Second failure while testing %s\n%s\n" % (db, str(exc))
                    msg += "Giving up Now."
                    self.logger.error(msg)
                    raise exc
                msg = "Database %s successfully created" % db
                self.logger.error(msg)
        except Exception as ex:
            msg = "General Exception while trying to connect to : %s\n%s" % (db, str(ex))
            self.logger.error(msg)
            raise ex



#!/usr/bin/env python
"""
_Core_

Core Database APIs


"""
from copy import copy

from Utils.IteratorTools import grouper
import WMCore.WMLogging
from WMCore.DataStructs.WMObject import WMObject
from WMCore.Database.ResultSet import ResultSet

class DBInterface(WMObject):
    """
    Base class for doing SQL operations using a SQLAlchemy engine, or
    pre-exisitng connection.

    processData will take a (list of) sql statements and a (list of)
    bind variable dictionaries and run the statements on the DB. If
    necessary it will substitute binds into the sql (MySQL).

    TODO:
        Add in some suitable exceptions in one or two places
        Test the hell out of it
        Support executemany()
    """

    logger = None
    engine = None

    def __init__(self, logger, engine):
        self.logger = logger
        self.logger.info ("Instantiating base WM DBInterface")
        self.engine = engine
        self.maxBindsPerQuery = 500

    def buildbinds(self, sequence, thename, therest=[{}]):
        """
        Build a list of binds. Can be used recursively, e.g.:
        buildbinds(file, 'file', buildbinds(pnn, 'location'), {'lumi':123})
        TODO: replace with an appropriate map function
        """
        binds = []
        for r in sequence:
            for i in self.makelist(therest):
                thebind = copy(i)
                thebind[thename] = r
                binds.append(thebind)
        return binds

    def executebinds(self, s=None, b=None, connection=None,
                     returnCursor=False):
        """
        _executebinds_

        returns a list of sqlalchemy.engine.base.ResultProxy objects
        """
        if b == None:
            resultProxy = connection.execute(s)
        else:
            resultProxy = connection.execute(s, b)

        if returnCursor:
            return resultProxy

        result = ResultSet()
        result.add(resultProxy)
        resultProxy.close()
        return result

    def executemanybinds(self, s=None, b=None, connection=None,
                         returnCursor=False):
        """
        _executemanybinds_
        b is a list of dictionaries for the binds, e.g.:

        b = [ {'bind1':'value1a', 'bind2': 'value2a'},
        {'bind1':'value1b', 'bind2': 'value2b'} ]

        see: http://www.gingerandjohn.com/archives/2004/02/26/cx_oracle-executemany-example/

        Can't executemany() selects - so do each combination of binds here instead.
        This will return a list of sqlalchemy.engine.base.ResultProxy object's
        one for each set of binds.

        returns a list of sqlalchemy.engine.base.ResultProxy objects
        """

        s = s.strip()
        if s.lower().endswith('select', 0, 6):
            """
            Trying to select many
            """
            if returnCursor:
                result = []
                for bind in b:
                    result.append(connection.execute(s, bind))
            else:
                result = ResultSet()
                for bind in b:
                    resultproxy = connection.execute(s, bind)
                    result.add(resultproxy)
                    resultproxy.close()

            return self.makelist(result)

        """
        Now inserting or updating many
        """
        result = connection.execute(s, b)
        return self.makelist(result)

    def connection(self):
        """
        Return a connection to the engine (from the connection pool)
        """
        return self.engine.connect()


    def processData(self, sqlstmt, binds={}, conn=None,
                    transaction=False, returnCursor=False):
        """
        set conn if you already have an active connection to reuse
        set transaction = True if you already have an active transaction

        """
        connection = None
        try:
            if not conn:
                connection = self.connection()
            else:
                connection = conn

            result = []
            # Can take either a single statement or a list of statements and binds
            sqlstmt = self.makelist(sqlstmt)
            binds = self.makelist(binds)
            if len(sqlstmt) > 0 and (len(binds) == 0 or (binds[0] == {} or binds[0] == None)):
                # Should only be run by create statements
                if not transaction:
                    #WMCore.WMLogging.sqldebug("transaction created in DBInterface")
                    trans = connection.begin()

                for i in sqlstmt:
                    r = self.executebinds(i, connection=connection,
                                          returnCursor=returnCursor)
                    result.append(r)

                if not transaction:
                    trans.commit()
            elif len(binds) > len(sqlstmt) and len(sqlstmt) == 1:
                #Run single SQL statement for a list of binds - use execute_many()
                if not transaction:
                    trans = connection.begin()
                for subBinds in grouper(binds, self.maxBindsPerQuery):
                    result.extend(self.executemanybinds(sqlstmt[0], subBinds,
                                                        connection=connection, returnCursor=returnCursor))

                if not transaction:
                    trans.commit()
            elif len(binds) == len(sqlstmt):
                # Run a list of SQL for a list of binds
                if not transaction:
                    trans = connection.begin()

                for i, s in enumerate(sqlstmt):
                    b = binds[i]

                    r = self.executebinds(s, b, connection=connection,
                                          returnCursor=returnCursor)
                    result.append(r)

                if not transaction:
                    trans.commit()
            else:
                self.logger.exception(
                    "DBInterface.processData Nothing executed, problem with your arguments")
                self.logger.exception(
                    "DBInterface.processData SQL = %s" % sqlstmt)
                WMCore.WMLogging.sqldebug('DBInterface.processData  sql is %s items long' % len(sqlstmt))
                WMCore.WMLogging.sqldebug('DBInterface.processData  binds are %s items long' % len(binds))
                assert_value = False
                if len(binds) == len(sqlstmt):
                    assert_value = True
                WMCore.WMLogging.sqldebug('DBInterface.processData are binds and sql same length? : %s' % (assert_value))
                WMCore.WMLogging.sqldebug('sql: %s\n binds: %s\n, connection:%s\n, transaction:%s\n' %
                                           (sqlstmt, binds, connection, transaction))
                WMCore.WMLogging.sqldebug('type check:\nsql: %s\n binds: %s\n, connection:%s\n, transaction:%s\n' %
                                           (type(sqlstmt), type(binds), type(connection), type(transaction)))
                raise Exception("""DBInterface.processData Nothing executed, problem with your arguments
                Probably mismatched sizes for sql (%i) and binds (%i)""" % (len(sqlstmt), len(binds)))
        finally:
            if not conn and connection != None:
                connection.close() # Return connection to the pool
        return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# pylint: disable=R0903
# R0903: too-few-public-methods
"""
File       : parseUnifiedCampaigns.py
Author     : Valentin Kuznetsov <vkuznet AT gmail dot com>
Description: Parse Unified campaigns json record or read them
from Unified MongoDB. The yielded campaign records are ready
for CouchDB and follow the campaign schema:
```
{
  "campaign_A": [record1, record2],
  "campaign_B: [...]
}
```
where individual records have the following structure:
```
{
 "CampaignName": "blah2019",
 "PrimaryAAA": boolean,
 "SecondaryAAA": boolean,
 "SiteWhiteList": ["T1", "T2"],
 "SiteBlackList": [...],
 "SecondaryLocation": ["T1", "T2"] # list with sites to hold a whole copy of the dataset
 "Secondaries": {
    "datasetA": ["list of sites for SiteWhitelist"],
    "datasetB": ["list of sites for SiteWhitelist"]
 },
 "MaxCopies": integer,
 "PartialCopy": integer,
 "TiersToDM": ["datatiers blocked in unified configuration that can be passed to DDM"]
}
```

***** Example of execution *****
python parseUnifiedCampaigns.py --dburi=mongodb://localhost:27017
       --dbname=unified --dbcoll=campaignsConfiguration --verbose=10 --fout=output.json

or, parsing campaigns.json file
python parseUnifiedCampaigns.py --fin=/path/WmAgentScripts/campaigns.json

or yet parsing a json file filled of campaign configurations already compliant
to the WMCore schema:
python parseUnifiedCampaigns.py --fin=wmcore_campaign.json --url=https://alancc7-cloud2.cern.ch/reqmgr2
         --verbose=10 --testcamp
"""


from builtins import object

import argparse
import json
import logging
# system modules
import os

# pymongo modules
try:
    from pymongo import MongoClient
except ImportError:
    print("pymongo library not found. Faking it")
    MongoClient = None

# WMCore modules
try:
    from WMCore.Services.ReqMgrAux.ReqMgrAux import ReqMgrAux
except ImportError:
    print("WMCore/ReqMgrAux environemnt not found. Faking it")
    ReqMgrAux = None


class OptionParser(object):
    "Helper class to handle script options"

    def __init__(self):
        "User based option parser"
        self.parser = argparse.ArgumentParser(prog='PROG')
        self.parser.add_argument("--fin", action="store",
                                 dest="fin", default="", help="Input file")
        self.parser.add_argument("--fout", action="store",
                                 dest="fout", default="",
                                 help="Output json file with all the campaigns created")
        self.parser.add_argument("--verbose", action="store",
                                 dest="verbose", default=0, help="verbosity level")
        self.parser.add_argument("--url", action="store",
                                 dest="url", default="",
                                 help="url of server where we should upload campaign configs")
        self.parser.add_argument("--dburi", action="store",
                                 dest="dburi", default="", help="MongoDB URI")
        self.parser.add_argument("--dbname", action="store",
                                 dest="dbname", default="", help="MongoDB database")
        self.parser.add_argument("--dbcoll", action="store",
                                 dest="dbcoll", default="", help="MongoDB collection")
        self.parser.add_argument("--testcamp", action="store_true",
                                 dest="testcamp", default=False,
                                 help="Insert test campaign configurations into Couch")


def intersect(slist1, slist2):
    "Helper function to intersect values from two non-empty lists"
    if slist1 and slist2:
        return list(set(slist1) & set(slist2))
    if slist1 and not slist2:
        return slist1
    if not slist1 and slist2:
        return slist2
    return []


def union(slist1, slist2):
    "Helper function to union values from two lists"
    if slist1 and slist2:
        return list(set(slist1) | set(slist2))
    if slist1 and not slist2:
        return slist1
    if not slist1 and slist2:
        return slist2
    return list(set(slist1 + slist2))


def getSecondaryAAA(initialValue, uniRecord):
    """
    SecondaryAAA: boolean to flag whether to use AAA for the secondary dataset;
    mapped from the secondary_AAA key, which can be either:
      * at top level or
      * under the secondaries dictionary.
    If it appears multiple times, we make an OR of the values.
    """
    for _, innerDict in uniRecord.get("secondaries", {}).items():
        if "secondary_AAA" in innerDict:
            print("Found internal secondary_AAA for campaign: %s" % uniRecord['name'])
            initialValue = initialValue or innerDict["secondary_AAA"]
    return initialValue


def getSiteList(keyName, initialValue, uniRecord):
    """
    Parse information related to the SiteWhiteList and SiteBlackList, which corresponds
    to a list of sites where the workflow gets (or doesn't get) assigned to.
    Mapped from multiple places in the Unified campaigns, such as:
      * top level dict,
      * under the parameters key and
      * under the secondaries dictionary (no longer used, see WMCore #10332).
    If it appears multiple times, we make an intersection of the values
    """
    if keyName in uniRecord.get("parameters", {}):
        print("Found internal %s for campaign: %s" % (keyName, uniRecord['name']))
        initialValue = intersect(initialValue, uniRecord["parameters"][keyName])

    return initialValue


def getSecondaryLocation(initialValue, uniRecord):
    """
    SecondaryLocation: list of sites where the secondary PREMIX dataset has to be placed
    as a whole (not necessarily also assigned to those).
    mapped from the SecondaryLocation key, which can be either:
      * at top level or
      * under the secondaries dictionary.
    If it appears multiple times, we make an intersection of the values.
    """
    for _, innerDict in uniRecord.get("secondaries", {}).items():
        if "SecondaryLocation" in innerDict:
            print("Found internal SecondaryLocation for campaign: %s" % uniRecord['name'])
            initialValue = intersect(initialValue, innerDict["SecondaryLocation"])
    return initialValue


def getSecondaries(initialValue, uniRecord):
    """
    Secondaries: dictionary with a map of allowed secondary datasets and where they are
    supposed to be placed (in conjunction with the top level location parameters;
    mapped from the secondaries top level key

    Each dataset will have a list value type, and the content is either:
      * taken from the SiteWhitelist key or
      * taken from the SecondaryLocation one
    """
    for dset, innerDict in uniRecord.get("secondaries", {}).items():
        print("Found secondaries for campaign: %s" % uniRecord['name'])
        initialValue[dset] = intersect(innerDict.get("SiteWhitelist", []),
                                       innerDict.get("SecondaryLocation", []))

    return initialValue


def parse(istream, verbose=0):
    "Parse Unified campaign configuration"
    wmCampaigns = []

    # re-map Unified keys into campaign schema ones
    remap = {
        'name': 'CampaignName',
        'SiteWhitelist': 'SiteWhiteList',
        'SiteBlacklist': 'SiteBlackList',
        'primary_AAA': 'PrimaryAAA',
        'secondary_AAA': 'SecondaryAAA',
        'SecondaryLocation': 'SecondaryLocation',
        'secondaries': 'Secondaries',
        'partial_copy': 'PartialCopy',
        'toDDM': 'TiersToDM',
        'maxcopies': 'MaxCopies'}
    # campaign schema dict
    confRec = {
        'CampaignName': None,
        'SiteWhiteList': [],
        'SiteBlackList': [],
        'PrimaryAAA': False,
        'SecondaryAAA': False,
        'SecondaryLocation': [],
        'Secondaries': {},
        'PartialCopy': 1,
        'TiersToDM': [],
        'MaxCopies': 1}

    if not isinstance(istream, list):
        istream = [istream]
    for rec in istream:
        if verbose > 1:
            print("read record: %s (type=%s)" % (rec, type(rec)))

        conf = dict(confRec)
        # Set default value from top level campaign configuration
        # or use the default values defined above
        for uniKey, wmKey in remap.items():
            conf[wmKey] = rec.get(uniKey, conf[wmKey])

        conf['SiteWhiteList'] = getSiteList("SiteWhitelist", conf['SiteWhiteList'], rec)
        conf['SiteBlackList'] = getSiteList("SiteBlacklist", conf['SiteBlackList'], rec)
        conf['SecondaryAAA'] = getSecondaryAAA(conf['SecondaryAAA'], rec)
        conf['SecondaryLocation'] = getSecondaryLocation(conf['SecondaryLocation'], rec)
        conf['Secondaries'] = getSecondaries(conf['Secondaries'], rec)
        if verbose:
            print("Final WMCore Campaign configuration: %s" % conf)
        wmCampaigns.append(conf)

    print("\nParser found and created %d campaign configurations\n" % len(wmCampaigns))
    return wmCampaigns


def process(recs):
    "Helper function to process given records and return unique list"
    rdict = {}
    for rec in recs:
        campaign = rec['CampaignName']
        if campaign in rdict:
            if rdict[campaign] != rec:
                print("WARNING: found new record for campaign '%s'" % campaign)
                print("new record: %s" % rec)
                print("old record: %s" % rdict[campaign])
        else:
            rdict[campaign] = rec
            yield rec


def upload(mgr, campRec):
    "Upload campaign configuration"
    if not mgr:
        return
    campaign = campRec.get('CampaignName', '')
    print("Inserting campaign record: %s" % json.dumps(campRec))
    if campaign:
        res = mgr.postCampaignConfig(campaign, campRec)
        print(res)
    else:
        print("ERROR: found a campaign record without a CampaignName value: %s" % campRec)


def insertTestCampaigns(mgr):
    """
    Insert a few campaigns configuration used for testing purposes
    NOTE: the campaign name might change according to the tests you're running.
    :param mgr: ReqMgrAux object
    """
    if not mgr:
        return

    defaultCamp = {'CampaignName': '', 'MaxCopies': 1, 'PartialCopy': 1, 'TiersToDM': [],
                   'PrimaryAAA': False, 'Secondaries': {}, 'SecondaryAAA': False,
                   'SecondaryLocation': ["T1_US_FNAL_Disk", "T2_CH_CERN"],
                   'SiteBlackList': [], 'SiteWhiteList': ["T1_US_FNAL", "T2_CH_CERN"]}

    testCamp = ("CMSSW_7_3_2__test2inwf-1510737328", "CMSSW_11_2_0_pre6__fullsim_noPU_2021_14TeV-1599843628",
                "RelVal_Generic_Campaign", "DMWM_Test", "Integ_Test",
                "Agent227_Val", "Agent229_Val", "Agent2211_Val",
                "Nov2023_Val", "Dec2023_Val", "Jan2024_Val", "Feb2024_Val", "Mar2024_Val"
                "HG2312_Val", "HG2401_Val", "HG2402_Val", "HG2403_Val")
    for campName in testCamp:
        defaultCamp['CampaignName'] = campName
        upload(mgr, defaultCamp)


def main():
    "Main function"
    optmgr = OptionParser()
    opts = optmgr.parser.parse_args()
    verbose = int(opts.verbose)
    logger = None
    mgr = None
    inputWMCore = False
    if verbose:
        logger = logging.getLogger('parse_campaign')
        logger.setLevel(logging.DEBUG)
        logging.basicConfig()
    if opts.url:
        key = os.getenv('X509_USER_KEY', '')
        cert = os.getenv('X509_USER_CERT', '')
        proxy = os.getenv('X509_USER_PROXY', '')
        if proxy and not cert:
            cert = proxy
            key = proxy
        hdict = {'cert': cert, 'key': key, 'pycurl': True}
        mgr = ReqMgrAux(opts.url, hdict, logger=logger)
    if opts.dburi:
        conn = MongoClient(host=opts.dburi)
        dbname = opts.dbname
        dbcoll = opts.dbcoll
        if verbose:
            print("### read data from '%s', %s/%s" % (opts.dburi, dbname, dbcoll))
        data = [r for r in conn[dbname][dbcoll].find()]
    elif opts.fin:
        fin = opts.fin
        if verbose:
            print("### read data from '%s'" % fin)
        data = []
        with open(fin, 'r') as istream:
            campData = json.load(istream)
            if isinstance(campData, dict):
                # then it's a Unified-like campaign schema
                for key, val in campData.items():
                    rec = {'name': key}
                    rec.update(val)
                    data.append(rec)
            elif isinstance(campData, list):
                # then the input file has WMCore-like campaign schema
                print("Found %d campaigns in the input file." % len(campData))
                data = campData
                inputWMCore = True
    if not inputWMCore:
        data = parse(data, verbose)

    output = []  # in case we want to dump all records to a json file
    for rec in process(data):
        output.append(rec)
        upload(mgr, rec)
    if opts.testcamp:
        insertTestCampaigns(mgr)
    if opts.fout:
        print("Saving all %d unique campaign records to: %s\n" % (len(output), opts.fout))
        with open(opts.fout, "w") as jo:
            json.dump(output, jo, indent=2)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
_Fileset_

Data object that contains a set of files

"""

from __future__ import print_function
from builtins import str, map

from operator import itemgetter
from WMCore.DataStructs.WMObject import WMObject
__all__ = []


class Fileset(WMObject):
    """
    _Fileset_
    Data object that contains a set of files
    """
    def __init__(self, name=None, files=None):
        """
        Assume input files are new
        """
        self.name = name
        self.files = set()

        if files is None:
            self.newfiles = set()
        else:
            self.newfiles = files

        # assume that the fileset is open at first
        self.open = True

        # assume that the lastUpdate of fileset is 0 at first
        self.lastUpdate = 0

    def setLastUpdate(self, timeUpdate):
        """
        _setLastUpdate_

        Change the last update time of this fileset.  The lastUpdate parameter is a int
        representing the last time where the fileset was modifed.
        """
        self.lastUpdate = timeUpdate

    def addFile(self, file):
        """
        Add a (set of) file(s) to the fileset
        If the file is already in self.files update that entry
            e.g. to handle updated location
        If the file is already in self.newfiles update that entry
            e.g. to handle updated location
        Else add the file to self.newfiles
        """
        file = self.makeset(file)
        new = file - self.getFiles(type='set')
        self.newfiles = self.makeset(self.newfiles) | new

        updated = self.makeset(file) & self.getFiles(type='set')
        "updated contains the original location information for updated files"

        self.files = self.files.union(updated)

    def getFiles(self, type='list'):
        if type == 'list':
            """
            List all files in the fileset - returns a set of file objects
            sorted by lfn.
            """
            files = list(self.getFiles(type='set'))

            try:
                files.sort(key=itemgetter('lfn'))
            except Exception as e:
                print('Problem with listFiles for fileset:', self.name)
                print(files.pop())
                raise e
            return files
        elif type == 'set':
            return self.makeset(self.files) | self.makeset(self.newfiles)
        elif type == 'lfn':
            """
            All the lfn's for files in the filesets
            """
            def getLFN(file):
                return file["lfn"]
            files = list(map(getLFN, self.getFiles(type='list')))
            return files
        elif type == 'id':
            """
            All the id's for files in the filesets
            """
            def getID(file):
                return file["id"]

            files = list(map(getID, self.getFiles(type='list')))
            return files

    def listNewFiles(self):
        """
        List all files in the fileset that are new - e.g. not in the DB - returns a set
        """
        return self.newfiles

    def commit(self):
        """
        Add contents of self.newfiles to self, empty self.newfiles
        """
        self.files = self.makeset(self.files) | self.makeset(self.newfiles)
        self.newfiles = set()

    def __len__(self):
        return len(self.getFiles(type='set'))

    def __iter__(self):
        for file in self.getFiles():
            yield file

    def markOpen(self, isOpen):
        """
        _markOpen_

        Change the open status of this fileset.  The isOpen parameter is a bool
        representing whether or not the fileset is open.
        """
        self.open = isOpen

    def __str__(self):
        """
        __str__

        Write out something useful
        """

        st = {'name': self.name, 'files': self.files}

        return str(st)
